#!/usr/bin/perl -w
#
# a crude frontend for dvbd to allow http-streaming
# to be started from xinetd, e.g. on port 8888.
#
# Stefan Seyfried, 2005
#
# MPEG-TS without time limit: mplayer http://hostname:8888/myprogram
# MPEG-TS for 10 minutes:     mplayer http://hostname:8888/myprogram#10
# MPEG-PS without time limit: mplayer http://hostname:8888/myprogram.mpg
# MPEG-PS for 10 minutes:     mplayer http://hostname:8888/myprogram.mpg#10
#
# http://servername:8888/list gives you a list of programs.
#
# todo: error handling (return 404 for unavailable channels)

# the type of your dvb-card. Can be "dvb-t", "dvb-s", "dvb-c"
my $type = "dvb-t";
#
# The priority of the stream; default priority of recordings is 10
# we stream with 9 so a scheduled recording will preempt the stream.
my $prio = "9";
#
# set this to "1" to always send a MPEG-PS.
my $ps = 0;

my $duration = 0;
my ($get,$line);

# crude HTTP-Request-parsing. Does not need any perl modules ;-)
$line = <>;
$line =~ m/^\S+\s(\S+)\s/; # GET /Program HTTP/1.1
$get = $1;                 # /Program

$request = unescape_uri($get); # %20...
$request =~ s#^/##;            # Program

# if a "#nnn" is last, nnn is the number of minutes to stream
if ( $request =~ m/(.*)#(\d+)$/ ) {
    $request  = $1;
    $duration = $2;
}


if ( $request =~ s#\.mpg$## ) {
    $ps = 1;
}

# the basic command
my @command=("dvbcat", "-p", $prio);

# do we have a timeout?
if ($duration != 0) {
    push (@command, "-d", $duration);
}

# send a program stream?
if ($ps != 0) {
    push (@command, "-P");
}

push (@command, $type, $request);

# HTTP Headers
print "HTTP/1.1 200 OK\n";
my $date=gmtime() . ' GMT';
print "Date: $date\n";
print "Server: dvbcat-network (perl,seife)\n";
print "Connection: close\n";

if ($request eq "list") {
    print "Content-Type: text/plain\n";
    open (LIST, "-|", "dvblist", "-p", $prio, $type) or die_in_header();

    my $list = "# ";
    while ($line=<LIST>) {
        $list .= $line;
    }
    print "Content-Length: " . length($list) . "\n\n";
    print $list;
} else {
    print "Content-Type: video/mpeg\n\n";

    exec @command;
}

#####################################################################
sub unescape_uri {
    my ($uri) = @_;
    $uri =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
    return $uri;
} 
###############################
sub die_in_header {
    print "\ndied in headers.\n";
    exit 1;
}

