#!/usr/bin/perl -wT
#
# a crude CGI for dvbd to allow http-streaming
# to be started from apache (tested with apache2)
#
# Stefan Seyfried, 2005
# Released under the GNU GPL v2
#
# parameters (append to the URL with '?', separate with ';')
# ?program=  Program name as listed in channels.conf
#            If no program is selected, the a list of channels is shown
# ?type=     [s,t,c], selects DVB type, if no type is given, the guide
#            is guessed from "card0 =" of /etc/dvbd/config
# ?prio=     priority for the stream, defaults to 9
# ?stype=    Stream type. '.mpg' for MPEG-PS, '.ts' for MPEG-TS.
#            the default depends on the name of this cgi. If the name ends
#            in '.mpg', the default is .mpg, otherwise it is .ts.
# ?duration= How long the stream should last. efaults to 0 => no limit
#
# todo: error handling (return an appropriate error for unavailable channels)

use strict;
use CGI qw(:standard);
my ($program, $type, $prio, $stype, $duration, $filter, $ttype);

# set $PATH secure
$ENV{'PATH'}="/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin";

$program  = param("program");
$type     = param("type");
$prio     = param("prio");
$stype    = param("stype");
$duration = param("time");
$filter   = param("filter");

# defaults
$prio += 0;
$prio = 9   if ($prio == 0);

if (!defined($type)) {
    my (@config, $line);
    # try to guess from dvbd config...
    if (open (CFG, "< /etc/dvbd/config")) {
        @config = <CFG>;
        @config = grep(/^card0/, @config);
        if (@config) {
            $line = $config[0];
            $line =~ m/^card0\s*=\s*dvb-([cst]):.*$/;
            $type = $1;
        }
    }
}

# un-taint $type
$type = lc($type);
if ($type eq "c") {
    $type = "c";
} elsif ($type eq "s") {
    $type = "s";
} else {
    $type = "t";
}

if (!defined($stype)) {
    if ($0 =~ m#\.mpg$#) {
        $stype = ".mpg";
    } else {
        $stype = ".ts";
    }
}

# stream on...
sub stream() {
    # priority, default: 9
    if (defined($prio)) {
        $prio =~ /(\d+)/;
        $prio = $1;
    }
    $prio = 9   if !defined($prio);

    # the basic command
    my @command=("/usr/bin/dvbcat", "-p", $prio);

    # do we have a timeout?
    if (defined($duration)) {
        $duration =~ /(\d+)/;
        $duration = $1;
        if (defined($duration)) {
            push (@command, "-d", $duration);
        }
    }

    # if we are named *.mpg assume that mpeg-ps should be default,
    # else consider $stype
    if ((!defined($stype) && ($0 =~ m#\.mpg$#)) ||
        (defined($stype) && ($stype eq ".mpg"))) {
        push (@command, "-P");
    }

    if (defined($program)) {
        $program =~ /(.*)/; # un-taint $program
        $program = $1;
        $ENV{'PATH'} = '';
        push (@command, "dvb-$type", $program);
        print "Content-Disposition: inline; filename=\"$program$stype\"\n";
        print "Content-Type: video/mpeg; name=\"$program$stype\";\n\n";
        exec @command;
    } else {
        # TODO: fix this, better error message.
        my $error = "400 Bad Request (Missing Program Name)";
        print header(-status=>$error),
              start_html($error),
              h2($error),
              end_html;
    }
}

# let's select a program to stream
sub print_menu() {
    my ($head, $count, $list, $line);
    print header(), start_html("DVB-$type Channel List");

    print start_form,
          "Parameters:",
          popup_menu(-name=>'type',
                     -values=>['s','t','c'],
                     -default=>$type,
                     -labels=>{'s'=>'DVB-S','t'=>'DVB-T','c'=>'DVB-C'}),
          popup_menu(-name=>"stype",
                     -values=>['.mpg','.ts'],
                     -default=>$stype,
                     -labels=>{'.mpg'=>'MPEG-PS','.ts'=>'MPEG-TS'}),
          " priority:",
          textfield(-name=>'prio',
                    -default=>$prio,
                    -size=>2),
          " duration (minutes):",
          textfield(-name=>'time',
                    -default=>$duration,
                    -size=>3),
          p,
          "Filter:",
          textfield(-name=>'filter', 
                    -default=>$filter),
          submit;

    if (!open (LIST, "-|", "/usr/bin/dvblist", "-p", $prio, "dvb-$type")) {
        print "Error: could not run dvblist";
        print end_html;
        exit 0;
    }

    $count = 0;

    $head = <LIST>;
    $list = "";
    while ($line = <LIST>) {
        chomp $line;
        if ($line =~ m/$filter/i) {
            $list .= "<a href=\"?type=$type;program=$line";
            # to keep the URL short, omit the defaults
            if ($prio != 9) {
                $list .= ";prio=$prio";
            }
            if (defined($duration)) {
                $list .= ";time=$duration";
            }
            $list .= ";stype=$stype\">$line</a><br>\n";
            $count++;
        }
    }

    print h2("$count $head");
    print $list;
    print end_html();
}


if (!defined($program)) {
    print_menu;
} else {
    stream;
}
