#!/usr/bin/perl

#   Copyright 2004 John Knottenbelt
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
#

#
# Possibilities to speed this up:
#  1. use libxml-libxslt-perl and mod_perl for apache (keep the compiled stylesheet)
#

use CGI;
use File::Temp qw/:POSIX/;
use Time::Local;

$recordingPath = "/home/jak/recordings";

%xmltvToDVB = ( "bbc1.bbc.co.uk" => "dvb-t/BBC ONE",
		"bbc2.bbc.co.uk" => "dvb-t/BBC TWO",
		"choice.bbc.co.uk" => "dvb-t/BBC THREE",
		"bbc3.bbc.co.uk" => "dvb-t/BBC THREE",
		"knowledge.bbc.co.uk" => "dvb-t/BBC FOUR",
		"news-24.bbc.co.uk" => "dvb-t/BBC NEWS 24", 
		"parliament.bbc.co.uk" => "dvb-t/BBC PARLMNT",
		"carlton.com" => "dvb-t/ITV1",
		"itv2.itv.co.uk" => "dvb-t/ITV2",
		"channel4.com" => "dvb-t/Channel 4",
		"channel5.co.uk" => "dvb-t/five" );

# Maximum number of days into the future
$maxoffset = 31;

sub listings {
  my $offset = shift;
  return "listings.$offset.xml" 
}

sub lookupXmlTV {
  my ($channel) = @_;

  for my $k (keys %xmltvToDVB) {
    if ($channel =~ m/^$k/) {
      return $xmltvToDVB{$k};
    }
  }
  undef;
}

sub printScheduled ($$$) {
    my $output = shift;
    my $schedule = shift;
    my $element = shift;

    print $output "  <$element>\n";
    foreach my $r (@$schedule) {
	# <programme start="20040102011500 +0000" channel="bbc2.bbc.co.uk"/>
	print $output "    <programme start=\"$r->{start}\" channel=\"$r->{channel}\"/>\n";
    }
    print $output "  </$element>\n";
}

sub toIsoDateTime ($) {
    my $time = shift;
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($time);
    return sprintf("%04d%02d%02d%02d%02d%02d", 
		   1900 + $year,$mon + 1,$mday, $hour,$min,$sec);
    
}

sub printFile($$) {
    my ($output, $filename) = @_;

    open INPUT, "<$filename"
	or return undef;
    
    while (<INPUT>) {
	print $output $_;
    }
    close INPUT;
}

sub printXml ($$$$) {
    my ($output, $scheduled, $recordings, $offset) = @_;

    open LISTING, "<" . listings($offset) 
	or die "Failed to open listings.xml: $!";
    
    $line = <LISTING>;
    print $output $line;

    $line = <LISTING>;
    print $output $line;

    my $now = toIsoDateTime(time);
    print $output "<root>\n";
    print $output "  <now>$now</now>\n";
    print $output "  <offset>$offset</offset>\n";

    printDays($output);
    printFile($output, "favourites.xml");
    printScheduled($output, $scheduled, "scheduled");
    printScheduled($output, $recordings, "recordings");

    while (<LISTING>) {
	print $output $_;
    }
    close LISTING;
    print $output "</root>\n";
}

sub printDays ($) {
    my ($output) = @_;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    my @days = ("Sun", "Mon", "Tue", "Wed", "Thur", "Fri", "Sat");

    # Earlier hours count as yesterday
    $wday = ($wday + 6) % 7 if ($hour < 6);

    for (my $o = 0; $o <= $maxoffset; $o++) {
	my $day = $days[($wday + $o) % 7];
	print $output "<day offset=\"$o\">$day</day>\n";
    }
}

sub printHtml ($$$) {
    my ($scheduled, $recordings, $offset) = @_;

    my $temp = tmpnam();
    open PIPE, "|/usr/bin/xsltproc --novalid tv.xsl - >$temp"
	or die "Failed to open XSLT pipe: $!\n";
    printXml(PIPE, $scheduled, $recordings, $offset);
    close PIPE;
    
    open TEMP, "<$temp"
	or die "Failed to open temp file: $temp: $!\n";
    while (<TEMP>) {
	print;
    }
    close TEMP;
    unlink $temp;

    # For debugging, save the XML to /tmp/state.xml
    open DEBUG, ">/tmp/state.xml"
	or die "Failed to open debug file state.xml: $!\n";
    printXml(DEBUG, $scheduled, $recordings, $offset);
    close DEBUG;
}

sub unescapeWS {
    my $arg = shift;
    # Unescape the WS
    $arg =~ s%\\_% %g;
    $arg =~ s%\\\\%\\%g;
    return $arg;
}

sub getScheduled {
    my @scheduled = ();
    open REC, "dvblist -h|";
    while (<REC>) {
	if (/^ (\d+) (\d+) (\S+) (\S+) (\S+) (\S+) (\S+) (\S+) (\S+) (\S+) \[(\S+)\]$/) {
	    my $job = $1;
	    my $path = unescapeWS($10);
	    my $extra = unescapeWS($11);

	    if ($extra =~ m%^(\S+) - (.+)$%) {

		my $channel = $1;
		my $start = "$2";

		my $r = { job => $job,
			  path => $path,
			  extra => $extra,
			  start => $start,
			  channel => $channel };

		push @scheduled, $r;
	    }
	}
    }
    close REC;
    return \@scheduled;
};

sub getRecordings {
    my @recordings = ();
    open REC, "dvblist -c|";
    while (<REC>) {
	if (/(\d+) (\d+) (\S+) (\S+) (\S+) (\S+) (\S+) (\S+) (\S+) (\S+) \[(\S+)\]$/) {
	    my $job = $1;
	    my $path = unescapeWS($10);
	    my $extra = unescapeWS($11);

	    if ($extra =~ m%^(\S+) - (.+)$%) {

		my $channel = $1;
		my $start = "$2";

		my $r = { job => $job,
			  path => $path,
			  extra => $extra,
			  start => $start,
			  channel => $channel };

		push @recordings, $r;
	    }
	}
    }
    close REC;
    return \@recordings;
};

sub alreadyScheduled {
    my ($schedule, $channel, $start) = @_;

    foreach my $r (@$schedule) {
	# <programme start="20040102011500 +0000" channel="bbc2.bbc.co.uk"/>
	return 1 if ($r->{start} eq $start &&
		     $r->{channel} eq $channel) ;
    }
    
    return undef;
}

sub toSeconds {
    my $iso = shift;
    
    if ($iso =~ /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/) {
	my $sec = $6;
	my $min = $5;
	my $hour = $4;
	my $mday = $3;
	my $mon = $2 - 1;
	my $year = $1 - 1900;
	return timelocal($sec,$min,$hour,$mday,$mon,$year);
    }
}

sub calcDuration {
    my ($start, $stop) = @_;
    my $startSec = toSeconds($start);
    my $stopSec = toSeconds($stop);
    return $stopSec - $startSec;
}

sub toPosixDateTime {
    my $iso = shift;

    if ($iso =~ /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/) {
	my $sec = $6;
	my $min = $5;
	my $hour = $4;
	my $mday = $3;
	my $mon = $2;
	my $year = $1;
	return "$hour:$min $year-$mon-$mday";
    }
}

sub cancel {
    my ($schedule, $recordings, $channel, $title, $subtitle, $start, $stop) = @_;

    foreach my $r (@$schedule, @$recordings) {
	# <programme start="20040102011500 +0000" channel="bbc2.bbc.co.uk"/>
	if ($r->{start} eq $start && $r->{channel} eq $channel) {
	    system("dvbsched -r $r->{job}");
	    return 1;
	}
    }
    return undef;
}

# Certain characters are invalid in a filename
# For example '/' which is the directory separator
# In order for the permissions check in dvbd to
# pass we need to remove any unintentional /

sub makeValidFilename {
    my $a = shift;
    $a =~ s%/%:%g;		# / is illegal in filenames
    return $a;
}

sub quoteSpaceWord {
    my ($word) = @_;
    return ($word =~ m/ / ? "'" . $word . "'" : $word);
}

sub schedule {
    my ($schedule, $channel, $title, $subtitle, $start, $stop) = @_;
    my $duration = calcDuration($start, $stop);
    
    if (!alreadyScheduled($schedule, $channel, $start)) {
	# print "Scheduling: [$channel] [$xmltvToDVB{$channel}] $title ($subtitle)\n";
	my $channelspec = lookupXmlTV($channel);
	if ($channelspec && $channelspec =~ m%^(.*)/(.*)$%) {
	    my $dvbType = $1;
	    my $dvbChannel = $2;

	    # Start the recording 1 minute early
	    # Duration is 4 minutes longer

	    my $startTime = toPosixDateTime(toIsoDateTime(toSeconds($start) - 60));
	    my $dur = int (($duration + 240 + 59) / 60);

	    my $oktitle = $title;
	    my $outputFile = "$recordingPath/" . makeValidFilename($title);
	    if ($subtitle ne "") {
		$outputFile .= " - " . makeValidFilename($subtitle);
	    }

	    $outputFile .= ".mpg";

	    my @command = ("dvbsched", 
#			   "-P", 
			   "-e", "$channel - $start",
			   "-n",
			   "$dvbType", "$dvbChannel", 
			   "$startTime", "$dur", 
			   "$outputFile");

	    # print "@command\n";
	    system(@command) and
		print "Failed: " . join(" ", map(quoteSpaceWord($_), @command)) . "\n $!\n";
	    return 1;
	}
	else {
	    print "Fatal! Trying to schedule $title: ";
	    print "No mapping for XMLTV channel $channel\n";
	}
    }
    else {
	print "$title Already scheduled\n";
    }
};

sub main {
    my $q = new CGI;
    print $q->header;

    my $scheduled = getScheduled();
    my $recordings = getRecordings();
    my $offset = 0;

    my @names = $q->param;

    if ($q->param("record") =~ /^(.*)\|\|\|(.*)\|\|\|(.*)\|\|\|(.*)\|\|\|(.*)$/) {
	if (schedule($scheduled, $1, $2, $3, $4, $5)) {
	    $scheduled = getScheduled();
	    $recordings = getRecordings();
	}
    }

    if ($q->param("cancel") =~ /^(.*)\|\|\|(.*)\|\|\|(.*)\|\|\|(.*)\|\|\|(.*)$/) {
	if (cancel($scheduled, $recordings, $1, $2, $3, $4, $5)) {
	    $scheduled = getScheduled();
	    $recordings = getRecordings();
	}
    }

    if ($q->param("offset")) {
    	$offset = int($q->param("offset"));
	$offset = $maxoffset if ($offset > $maxoffset);
	$offset = 0 if ($offset < 0);
    }

    while ($maxoffset > 0 && ! -f listings($maxoffset)) {
	$maxoffset--;
    }

    printHtml($scheduled, $recordings, $offset);
}

sub test {
    printScheduled(STDOUT, getScheduled(), "scheduled");
    printScheduled(STDOUT, getRecordings(), "recordings");
}

main();
