<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:key name="sched" match="/root/scheduled/programme" use="concat(@start,@channel)"/>
<xsl:key name="rec" match="/root/recordings/programme" use="concat(@start,@channel)"/>
<xsl:key name="fav" match="/root/favourites/programme" use="@title"/>
<xsl:output method="html" indent="yes"/>

<xsl:template match="/">
  <html>
    <head>
      <title>TV Listings</title>
      <link rel="stylesheet" href="tv.css" type="text/css"/>
    </head>
    <body>
      <form method="POST" action="tv.cgi">
        <input type="hidden" name="offset" value="{/root/offset}"/>
        <xsl:apply-templates select="/root/tv"/>
      </form>
    </body>
  </html>
</xsl:template>

<!-- Main starting/ending HTML -->
<xsl:template match="tv">
  <h1>TV Listings</h1>
  <xsl:variable name="now" select="/root/now"/>

  <p>
  <xsl:call-template name="navbar"/>
  <h2 id="favourites">Favourites</h2>
  </p>
  <table class="programmes" id="fav">
    <xsl:apply-templates select="programme[substring(@stop,0,15) &gt;= $now and key('fav', title)]"/>
  </table>

  <p>
  <xsl:call-template name="navbar"/>
  <h2 id="next">Next</h2>
  </p>
  <table class="programmes" id="next">
    <xsl:apply-templates select="programme[substring(@start,0,15) &gt;= $now]"/>
  </table>
  
  <p>
  <xsl:call-template name="navbar"/>
  <h2 id="now">Already Started</h2>
  </p>
  <table class="programmes" id="now">
    <xsl:apply-templates select="programme[substring(@start,0,15) &lt;= $now and $now &lt; substring(@stop,0,15)]"/>
  </table>

  <p>
  <xsl:call-template name="navbar"/>
  <h2 id="scheduled">Scheduled</h2>
  </p>
  <table class="programmes" id="scheduled">
    <xsl:apply-templates select="programme[key('sched', concat(@start,@channel))]">
      <xsl:with-param name="scheduled">true</xsl:with-param>
    </xsl:apply-templates>
  </table>

  <p>
  <xsl:call-template name="navbar"/>
  <h2 id="recording">Recording</h2>
  </p>
  <table class="programmes" id="recording">
    <xsl:apply-templates select="programme[key('rec', concat(@start,@channel))]">
      <xsl:with-param name="scheduled">true</xsl:with-param>
  </xsl:apply-templates>
  </table>
</xsl:template>

<xsl:template name="navbar">
  <div class="navbar">
    <table class="navbar">
      <tr class="navbar">
	<td>
	  <table class="navbar">
	    <tr class="navbar">
	      <td class="navbar"><a href="#favourites">Favourites</a></td>
	      <td class="navbar"><a href="#next">Next</a></td>
	      <td class="navbar"><a href="#now">Already Started</a></td>
	      <td class="navbar"><a href="#scheduled">Scheduled</a></td>
	      <td class="navbar"><a href="#recording">Recording</a></td>
	      <td class="navbar"><a href="">Refresh</a></td>
	    </tr>
	 </table>
       </td>
      </tr>
      <tr class="navbar">
	<td>
	  <table class="navbar" width="100%">
	    <tr class="navbar">
	      <xsl:call-template name="display-offset"/>
	    </tr>
	  </table>
	</td>
      </tr>
    </table>
  </div>
</xsl:template>

<!-- HTML for each 'programme' tag -->
<xsl:template match="tv/programme">
  <xsl:param name="scheduled">false</xsl:param>
  <tr class="r{position() mod 2}">
    <td class="date-time">
      <span class="date">
        <xsl:value-of select="substring(@start,7,2)"/>/<xsl:value-of select="concat(substring(@start,5,2),' ')"/>
      </span>
      <span class="time">
        <xsl:value-of select="substring(@start,9,2)"/>:<xsl:value-of select="substring(@start,11,2)"/>
      </span> - <span class="time"> <xsl:value-of select="substring(@stop,9,2)"/>:<xsl:value-of select="substring(@stop,11,2)"/> </span>
    </td>
    <xsl:choose>
      <xsl:when test="category='Film'">
        <div class="film">
          <td class="title">
            Film:
            <span class="film-title">
              <xsl:value-of select="title"/>
              <xsl:if test="sub-title">
                - <xsl:value-of select="sub-title"/>
              </xsl:if>
            </span>
            [<xsl:element name="a"><xsl:attribute name="href">http://www.imdb.com/Title?<xsl:value-of select="title"/>%20<xsl:if test="date">(<xsl:value-of select="date"/>)</xsl:if></xsl:attribute>imdb</xsl:element>]
          </td>
        </div>
      </xsl:when>
      <xsl:otherwise>
        <td class="title">
          <xsl:value-of select="title"/>
          <xsl:if test="sub-title">
            - <xsl:value-of select="sub-title"/>
          </xsl:if>
        </td>
      </xsl:otherwise>
    </xsl:choose>

    <div class="channel-{@channel}">
      <td class="channel">
        <span class="channel">
          <xsl:variable name="channel" select="@channel"/>
          <xsl:value-of select="/root/tv/channel[@id=$channel]/display-name[1]"/>
        </span>
      </td>
    </div>
  </tr>

  <tr class="r{position() mod 2}">
    <td class="button">
      <xsl:choose>
        <xsl:when test="$scheduled='true'">
          <button name="cancel" type="submit"
            value="{@channel}|||{title}|||{sub-title}|||{@start}|||{@stop}">Cancel</button>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="key('rec', concat(@start,@channel)) or key('sched', concat(@start,@channel))">
                <button name="cancel" type="submit"
                  value="{@channel}|||{title}|||{sub-title}|||{@start}|||{@stop}">Cancel</button>
                
              <p>
                <xsl:choose>
                  <xsl:when test="key('sched', concat(@start,@channel))">
                    <a href="#scheduled">Scheduled</a>
                  </xsl:when>
                  <xsl:otherwise>
                    <a href="#recording">Recording</a>
                  </xsl:otherwise>
                </xsl:choose>
              </p>
            </xsl:when>
            <xsl:otherwise>
              <button name="record" type="submit" 
                value="{@channel}|||{title}|||{sub-title}|||{@start}|||{@stop}">Record</button>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </td>
    <td colspan="2">
      <xsl:value-of select="desc"/>
    </td>
  </tr>
</xsl:template>

<xsl:template name="display-offset">
  <xsl:variable name="offset"><xsl:value-of select="/root/offset"/></xsl:variable>
  <xsl:for-each select="/root/day">
    <xsl:choose>
      <xsl:when test="number(@offset)=number($offset)">
        <td class="currentday">
          <xsl:value-of select="."/>
        </td>
      </xsl:when>
      <xsl:otherwise>
        <td class="navbar">
          <a href="?offset={@offset}"><xsl:value-of select="."/></a>
        </td>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>

